@@ -1,5 +1,5 @@
 #  shadowstep/page_object/page_object_generator.py
-
+import inspect
 import json
 import logging
 import os
@@ -50,21 +50,23 @@ class PageObjectGenerator:
         self.env = Environment(
             loader=FileSystemLoader(templates_dir),  # откуда загружать шаблоны (директория с .j2-файлами)
             autoescape=False,  # отключаем автоэкранирование HTML/JS (не нужно при генерации Python-кода)
-            keep_trailing_newline=True, # сохраняем завершающий перевод строки в файле (важно для git-diff, PEP8 и т.д.)
+            keep_trailing_newline=True,
+            # сохраняем завершающий перевод строки в файле (важно для git-diff, PEP8 и т.д.)
             trim_blocks=True,  # удаляет новую строку сразу после {% block %} или {% endif %} (уменьшает пустые строки)
-            lstrip_blocks=True # удаляет ведущие пробелы перед {% block %} (избавляет от случайных отступов и пустых строк)
+            lstrip_blocks=True
+            # удаляет ведущие пробелы перед {% block %} (избавляет от случайных отступов и пустых строк)
         )
         # добавляем фильтр repr
         self.env.filters['pretty_dict'] = _pretty_dict
 
     def generate(
-        self,
-        source_xml: str,
-        output_dir: str,
-        max_name_words: int = 5,
-        attributes: Optional[
-            Union[Set[str], Tuple[str], List[str]]
-        ] = None
+            self,
+            source_xml: str,
+            output_dir: str,
+            max_name_words: int = 5,
+            attributes: Optional[
+                Union[Set[str], Tuple[str], List[str]]
+            ] = None
     ):
         """
         Оркестратор:
@@ -104,20 +106,46 @@ class PageObjectGenerator:
         )
         properties: List[Dict] = []
 
-        # 5.1) обычные свойства
+        # 5.1)
+        anchor_pairs = self._find_anchor_element_pairs(elems)
+        self.logger.debug(f"{anchor_pairs=}")
+
+        # 5.2) обычные свойства
         for prop in self._build_regular_props(
-            elems,
-            title_el,
-            summary_pairs,
-            attr_list,
-            include_class,
-            max_name_words,
-            used_names,
-            recycler_id
+                elems,
+                title_el,
+                summary_pairs,
+                attr_list,
+                include_class,
+                max_name_words,
+                used_names,
+                recycler_id
         ):
             properties.append(prop)
 
-        # 5.2) summary-свойства
+        # 5.2.1) построим мапу id→имя свойства, чтобы потом найти anchor_name
+        self._anchor_name_map = {p['element_id']: p['name']
+                                 for p in properties
+                                 if 'element_id' in p}
+
+        # 5.3) switchers: собираем через общий _build_switch_prop
+        for anchor, switch, depth in anchor_pairs:
+            name, anchor_name, locator, depth = self._build_switch_prop(
+                anchor, switch, depth,
+                attr_list, include_class,
+                max_name_words, used_names
+            )
+            properties.append({
+                "name": name,
+                "locator": locator,
+                "sibling": False,
+                "via_recycler": switch.get("scrollable_parents", [None])[0] == recycler_id if switch.get(
+                    "scrollable_parents") else False,
+                "anchor_name": anchor_name,
+                "depth": depth,
+            })
+
+        # 5.4) summary-свойства
         for title_e, summary_e in summary_pairs:
             name, locator, summary_id, base_name = self._build_summary_prop(
                 title_e,
@@ -135,10 +163,10 @@ class PageObjectGenerator:
                 'base_name': base_name,
             })
 
-        # 5.3) удаляем дубликаты элементов
+        # 5.5) удаляем дубликаты элементов
         properties = self._filter_duplicates(properties)
 
-        # 5.4 )
+        # 5.6)
         need_recycler = any(p.get("via_recycler") for p in properties)
         recycler_locator = (
             self._build_locator(recycler_el, attr_list, include_class)
@@ -171,10 +199,10 @@ class PageObjectGenerator:
     # —————————————————————————————————————————————————————————————————————————
 
     def _prepare_attributes(
-        self,
-        attributes: Optional[
-            Union[Set[str], Tuple[str], List[str]]
-        ]
+            self,
+            attributes: Optional[
+                Union[Set[str], Tuple[str], List[str]]
+            ]
     ) -> Tuple[List[str], bool]:
         default = ['text', 'content-desc', 'resource-id']
         attr_list = list(attributes) if attributes else default.copy()
@@ -188,10 +216,10 @@ class PageObjectGenerator:
         return [p.lower() for p in parts if p]
 
     def _build_locator(
-        self,
-        el: Dict[str, str],
-        attr_list: List[str],
-        include_class: bool
+            self,
+            el: Dict[str, str],
+            attr_list: List[str],
+            include_class: bool
     ) -> Dict[str, str]:
         # loc: Dict[str, str] = {
         #     k: el[k] for k in attr_list if el.get(k)
@@ -221,15 +249,15 @@ class PageObjectGenerator:
 
     def _raw_title(self, title_el: Dict[str, str]) -> str:
         return (
-            title_el.get('text')
-            or title_el.get('content-desc')
-            or title_el.get('resource-id', '').split('/', 1)[-1]
+                title_el.get('text')
+                or title_el.get('content-desc')
+                or title_el.get('resource-id', '').split('/', 1)[-1]
         )
 
     def _format_names(self, raw_title: str) -> Tuple[str, str]:
         parts = re.split(r'[^\w]+', unidecode(raw_title))
         class_name = 'Page' + ''.join(p.capitalize() for p in parts if p)
-        file_name  = re.sub(
+        file_name = re.sub(
             r'(?<!^)(?=[A-Z])', '_', class_name
         ).lower() + '.py'
         return class_name, file_name
@@ -271,14 +299,14 @@ class PageObjectGenerator:
         return name, locator, summary_id, base_name
 
     def _build_regular_props(
-        self,
-        elems: List[Dict[str, str]],
-        title_el: Dict[str, str],
-        summary_pairs: List[Tuple[Dict[str, str], Dict[str, str]]],
-        attr_list: List[str],
-        include_class: bool,
-        max_name_words: int,
-        used_names: Set[str],
+            self,
+            elems: List[Dict[str, str]],
+            title_el: Dict[str, str],
+            summary_pairs: List[Tuple[Dict[str, str], Dict[str, str]]],
+            attr_list: List[str],
+            include_class: bool,
+            max_name_words: int,
+            used_names: Set[str],
             recycler_id
     ) -> List[Dict]:
         props: List[Dict] = []
@@ -302,7 +330,7 @@ class PageObjectGenerator:
             else:
                 raw = el.get(key) or self._strip_package_prefix(rid)
             words = self._slug_words(raw)[:max_name_words]
-            base   = "_".join(words) or key.replace('-', '_')
+            base = "_".join(words) or key.replace('-', '_')
             suffix = el.get('class', '').split('.')[-1].lower()
             raw_name = f"{base}_{suffix}"
 
@@ -315,9 +343,11 @@ class PageObjectGenerator:
 
             props.append({
                 'name': name,
+                'element_id': el['id'],
                 'locator': locator,
                 'sibling': False,
-                'via_recycler': el.get("scrollable_parents", [None])[0] == recycler_id if el.get("scrollable_parents") else False,
+                'via_recycler': el.get("scrollable_parents", [None])[0] == recycler_id if el.get(
+                    "scrollable_parents") else False,
             })
         #     self.logger.debug("=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=")
         #     self.logger.debug(f"{el.items()}")
@@ -400,3 +430,127 @@ class PageObjectGenerator:
         deepest = max(candidates, key=len)
         return deepest[0] if deepest else None
 
+    def _find_anchor_element_pairs(
+            self,
+            elements: List[Dict[str, Any]],
+            max_depth: int = 5,
+            target: Tuple[str, str] = ('class', 'android.widget.Switch')
+    ) -> List[Tuple[Dict[str, Any], Dict[str, Any], int]]:
+        """
+        Ищет тройки (anchor, target_element, depth), где:
+          - target_element — элемент, у которого атрибут target[0] содержит target[1]
+          - anchor         — соседний элемент с text или content-desc (вплоть до одного уровня вложенности)
+          - depth          — сколько раз поднялись по дереву до общего родителя
+        """
+        from collections import defaultdict
+
+        by_attr, attr_val = target
+
+        # 1) группировка: parent_id → прямые дети
+        children_by_parent: Dict[Optional[str], List[Dict[str, Any]]] = defaultdict(list)
+        for el in elements:
+            children_by_parent[el.get('parent_id')].append(el)
+
+        # id → элемент, для подъема вверх
+        el_by_id = {el['id']: el for el in elements if 'id' in el}
+
+        # собрать всех потомков по дереву (для проверки уникальности)
+        def collect_descendants(parent_id: str) -> List[Dict[str, Any]]:
+            stack = [parent_id]
+            result = []
+            while stack:
+                pid = stack.pop()
+                for child in children_by_parent.get(pid, []):
+                    result.append(child)
+                    stack.append(child['id'])
+            return result
+
+        pairs: List[Tuple[Dict[str, Any], Dict[str, Any], int]] = []
+
+        # 2) цикл по всем элементам-мишеням
+        for target_el in filter(lambda e: attr_val in e.get(by_attr, ''), elements):
+            current = target_el
+            depth = 0
+            anchor = None
+
+            # 2.a) поднимаемся вверх до max_depth
+            while depth <= max_depth and current.get('parent_id'):
+                parent_id = current['parent_id']
+                siblings = children_by_parent.get(parent_id, [])
+                siblings.sort(key=lambda x: int(x.get('index', 0)))
+
+                # 2.b) ищем anchor среди siblings и их детей
+                for sib in siblings:
+                    if sib is target_el:
+                        continue
+
+                    if sib.get('text') or sib.get('content-desc'):
+                        anchor = sib
+                        break
+
+                    for child in children_by_parent.get(sib['id'], []):
+                        if child.get('text') or child.get('content-desc'):
+                            anchor = child
+                            break
+                    if anchor:
+                        break
+
+                if anchor:
+                    # проверяем, что в subtree только один target
+                    subtree = collect_descendants(parent_id)
+                    count = sum(1 for el in subtree if attr_val in el.get(by_attr, ''))
+                    if count == 1:
+                        pairs.append((anchor, target_el, depth))
+                    else:
+                        self.logger.warning(
+                            f"Ambiguous targets under parent {parent_id}: {count} found. Skipping."
+                        )
+                    break
+
+                # идем к следующему родителю
+                current = el_by_id.get(parent_id, {})
+                depth += 1
+
+            if not anchor:
+                self.logger.debug(
+                    f"No anchor found for element {target_el.get('id')} up to depth {max_depth}"
+                )
+
+        self.logger.debug(f"Found anchor-element-depth triplets: {pairs}")
+        return pairs
+
+    def _build_switch_prop(
+            self,
+            anchor_el: Dict[str, Any],
+            switch_el: Dict[str, Any],
+            depth: int,
+            attr_list: List[str],
+            include_class: bool,
+            max_name_words: int,
+            used_names: Set[str]
+    ) -> Tuple[str, str, Dict[str, str], int]:
+        """
+        Возвращает кортеж:
+         - name         — имя свойства-свитчера
+         - anchor_name  — имя свойства-якоря (уже сгенерированного)
+         - locator      — словарь для get_element(switch_el)
+         - depth        — глубина подъёма (сколько раз get_parent())
+        """
+        # 1) имя якоря найдём в списке regular_props по id
+        anchor_name = self._anchor_name_map[anchor_el['id']]
+
+        # 2) генерим имя для switch
+        raw = anchor_el.get('text') or anchor_el.get('content-desc') or ""
+        words = self._slug_words(raw)[:max_name_words]
+        base = "_".join(words) or "switch"
+        name = self._sanitize_name(f"{base}_switch")
+        i = 1
+        while name in used_names:
+            name = self._sanitize_name(f"{base}_switch_{i}")
+            i += 1
+        used_names.add(name)
+
+        # 3) локатор для самого switch
+        locator = self._build_locator(switch_el, attr_list, include_class)
+
+        return name, anchor_name, locator, depth
